@@ -13,9 +13,7 @@
 #
 # Copyright Buildbot Team Members
 
-from future.utils import lrange
-
-ALL_RESULTS = lrange(7)
+ALL_RESULTS = list(range(7))
 SUCCESS, WARNINGS, FAILURE, SKIPPED, EXCEPTION, RETRY, CANCELLED = ALL_RESULTS
 Results = ["success", "warnings", "failure", "skipped", "exception", "retry", "cancelled"]
 
@@ -34,6 +32,7 @@ def worst_status(a, b):
     for s in (CANCELLED, RETRY, EXCEPTION, FAILURE, WARNINGS, SUCCESS, SKIPPED):
         if s in (a, b):
             return s
+    return None
 
 
 def computeResultAndTermination(obj, result, previousResult):
@@ -62,7 +61,7 @@ def computeResultAndTermination(obj, result, previousResult):
     return result, terminate
 
 
-class ResultComputingConfigMixin(object):
+class ResultComputingConfigMixin:
 
     haltOnFailure = False
     flunkOnWarnings = False
