@@ -13,8 +13,6 @@
 #
 # Copyright Buildbot Team Members
 
-from future.utils import text_type
-
 import os
 import pprint
 import re
@@ -51,7 +49,7 @@ class MasterShellCommand(BuildStep):
         self.interruptSignal = kwargs.pop('interruptSignal', 'KILL')
         self.logEnviron = kwargs.pop('logEnviron', True)
 
-        BuildStep.__init__(self, **kwargs)
+        super().__init__(**kwargs)
 
         self.command = command
         self.masterWorkdir = self.workdir
@@ -72,15 +70,14 @@ class MasterShellCommand(BuildStep):
                 self.step.stdio_log.addHeader(
                     "exit status %d\n" % status_object.value.exitCode)
             if status_object.value.signal is not None:
-                self.step.stdio_log.addHeader(
-                    "signal %s\n" % status_object.value.signal)
+                self.step.stdio_log.addHeader("signal {}\n".format(status_object.value.signal))
             self.step.processEnded(status_object)
 
     def start(self):
         # render properties
         command = self.command
         # set up argv
-        if isinstance(command, (text_type, bytes)):
+        if isinstance(command, (str, bytes)):
             if runtime.platformType == 'win32':
                 # allow %COMSPEC% to have args
                 argv = os.environ['COMSPEC'].split()
@@ -103,13 +100,13 @@ class MasterShellCommand(BuildStep):
 
         self.stdio_log = stdio_log = self.addLog("stdio")
 
-        if isinstance(command, (text_type, bytes)):
+        if isinstance(command, (str, bytes)):
             stdio_log.addHeader(command.strip() + "\n\n")
         else:
             stdio_log.addHeader(" ".join(command) + "\n\n")
         stdio_log.addHeader("** RUNNING ON BUILDMASTER **\n")
-        stdio_log.addHeader(" in dir %s\n" % os.getcwd())
-        stdio_log.addHeader(" argv: %s\n" % (argv,))
+        stdio_log.addHeader(" in dir {}\n".format(os.getcwd()))
+        stdio_log.addHeader(" argv: {}\n".format(argv))
         self.step_status.setText(self.describe())
 
         if self.env is None:
@@ -134,9 +131,9 @@ class MasterShellCommand(BuildStep):
             newenv = {}
             for key, v in env.items():
                 if v is not None:
-                    if not isinstance(v, (text_type, bytes)):
-                        raise RuntimeError("'env' values must be strings or "
-                                           "lists; key '%s' is incorrect" % (key,))
+                    if not isinstance(v, (str, bytes)):
+                        raise RuntimeError(("'env' values must be strings or "
+                                            "lists; key '{}' is incorrect").format(key))
                     newenv[key] = p.sub(subst, env[key])
             env = newenv
 
@@ -150,7 +147,7 @@ class MasterShellCommand(BuildStep):
 
     def processEnded(self, status_object):
         if status_object.value.signal is not None:
-            self.descriptionDone = ["killed (%s)" % status_object.value.signal]
+            self.descriptionDone = ["killed ({})".format(status_object.value.signal)]
             self.step_status.setText(self.describe(done=True))
             self.finished(FAILURE)
         elif status_object.value.exitCode != 0:
@@ -169,7 +166,7 @@ class MasterShellCommand(BuildStep):
             pass
         except error.ProcessExitedAlready:
             pass
-        BuildStep.interrupt(self, reason)
+        super().interrupt(reason)
 
 
 class SetProperty(BuildStep):
@@ -179,7 +176,7 @@ class SetProperty(BuildStep):
     renderables = ['property', 'value']
 
     def __init__(self, property, value, **kwargs):
-        BuildStep.__init__(self, **kwargs)
+        super().__init__(**kwargs)
         self.property = property
         self.value = value
 
@@ -197,7 +194,7 @@ class SetProperties(BuildStep):
     renderables = ['properties']
 
     def __init__(self, properties=None, **kwargs):
-        BuildStep.__init__(self, **kwargs)
+        super().__init__(**kwargs)
         self.properties = properties
 
     def run(self):
@@ -215,7 +212,7 @@ class Assert(BuildStep):
     renderables = ['check']
 
     def __init__(self, check, **kwargs):
-        BuildStep.__init__(self, **kwargs)
+        super().__init__(**kwargs)
         self.check = check
         self.descriptionDone = ["checked {}".format(repr(self.check))]
 
@@ -232,7 +229,7 @@ class LogRenderable(BuildStep):
     renderables = ['content']
 
     def __init__(self, content, **kwargs):
-        BuildStep.__init__(self, **kwargs)
+        super().__init__(**kwargs)
         self.content = content
 
     def start(self):
