@@ -24,7 +24,7 @@ from buildbot.steps.shell import ShellCommand
 class MaxQObserver(buildstep.LogLineObserver):
 
     def __init__(self):
-        buildstep.LogLineObserver.__init__(self)
+        super().__init__()
         self.failures = 0
 
     def outLineReceived(self, line):
@@ -39,8 +39,8 @@ class MaxQ(ShellCommand):
     def __init__(self, testdir=None, **kwargs):
         if not testdir:
             config.error("please pass testdir")
-        kwargs['command'] = 'run_maxq.py %s' % (testdir,)
-        ShellCommand.__init__(self, **kwargs)
+        kwargs['command'] = 'run_maxq.py {}'.format(testdir)
+        super().__init__(**kwargs)
         self.observer = MaxQObserver()
         self.addLogObserver('stdio', self.observer)
 
