@@ -13,12 +13,9 @@
 #
 # Copyright Buildbot Team Members
 
-from future.utils import text_type
-
 import json
 
 from twisted.internet import defer
-from twisted.internet import reactor
 from twisted.python import failure
 
 from buildbot.data import connector
@@ -59,22 +56,22 @@ class FakeUpdates(service.AsyncService):
     def assertProperties(self, sourced, properties):
         self.testcase.assertIsInstance(properties, dict)
         for k, v in properties.items():
-            self.testcase.assertIsInstance(k, text_type)
+            self.testcase.assertIsInstance(k, str)
             if sourced:
                 self.testcase.assertIsInstance(v, tuple)
                 self.testcase.assertEqual(len(v), 2)
                 propval, propsrc = v
-                self.testcase.assertIsInstance(propsrc, text_type)
+                self.testcase.assertIsInstance(propsrc, str)
             else:
                 propval = v
             try:
                 json.dumps(propval)
             except (TypeError, ValueError):
-                self.testcase.fail("value for %s is not JSON-able" % (k,))
+                self.testcase.fail("value for {} is not JSON-able".format(k))
 
     # update methods
 
-    def addChange(self, files=None, comments=None, author=None,
+    def addChange(self, files=None, comments=None, author=None, committer=None,
                   revision=None, when_timestamp=None, branch=None, category=None,
                   revlink='', properties=None, repository='', codebase=None,
                   project='', src=None):
@@ -84,15 +81,17 @@ class FakeUpdates(service.AsyncService):
         # double-check args, types, etc.
         if files is not None:
             self.testcase.assertIsInstance(files, list)
-            map(lambda f: self.testcase.assertIsInstance(f, text_type), files)
-        self.testcase.assertIsInstance(comments, (type(None), text_type))
-        self.testcase.assertIsInstance(author, (type(None), text_type))
-        self.testcase.assertIsInstance(revision, (type(None), text_type))
+            map(lambda f: self.testcase.assertIsInstance(f, str), files)
+        self.testcase.assertIsInstance(comments, (type(None), str))
+        self.testcase.assertIsInstance(author, (type(None), str))
+        self.testcase.assertIsInstance(committer, (type(None), str))
+        self.testcase.assertIsInstance(revision, (type(None), str))
         self.testcase.assertIsInstance(when_timestamp, (type(None), int))
-        self.testcase.assertIsInstance(branch, (type(None), text_type))
+        self.testcase.assertIsInstance(branch, (type(None), str))
 
         if callable(category):
             pre_change = self.master.config.preChangeGenerator(author=author,
+                                                               committer=committer,
                                                                files=files,
                                                                comments=comments,
                                                                revision=revision,
@@ -104,13 +103,13 @@ class FakeUpdates(service.AsyncService):
                                                                project=project)
             category = category(pre_change)
 
-        self.testcase.assertIsInstance(category, (type(None), text_type))
-        self.testcase.assertIsInstance(revlink, (type(None), text_type))
+        self.testcase.assertIsInstance(category, (type(None), str))
+        self.testcase.assertIsInstance(revlink, (type(None), str))
         self.assertProperties(sourced=False, properties=properties)
-        self.testcase.assertIsInstance(repository, text_type)
-        self.testcase.assertIsInstance(codebase, (type(None), text_type))
-        self.testcase.assertIsInstance(project, text_type)
-        self.testcase.assertIsInstance(src, (type(None), text_type))
+        self.testcase.assertIsInstance(repository, str)
+        self.testcase.assertIsInstance(codebase, (type(None), str))
+        self.testcase.assertIsInstance(project, str)
+        self.testcase.assertIsInstance(src, (type(None), str))
 
         # use locals() to ensure we get all of the args and don't forget if
         # more are added
@@ -119,7 +118,7 @@ class FakeUpdates(service.AsyncService):
         return defer.succeed(len(self.changesAdded))
 
     def masterActive(self, name, masterid):
-        self.testcase.assertIsInstance(name, text_type)
+        self.testcase.assertIsInstance(name, str)
         self.testcase.assertIsInstance(masterid, int)
         if masterid:
             self.testcase.assertEqual(masterid, 1)
@@ -127,7 +126,7 @@ class FakeUpdates(service.AsyncService):
         return defer.succeed(None)
 
     def masterStopped(self, name, masterid):
-        self.testcase.assertIsInstance(name, text_type)
+        self.testcase.assertIsInstance(name, str)
         self.testcase.assertEqual(masterid, 1)
         self.thisMasterActive = False
         return defer.succeed(None)
@@ -146,18 +145,17 @@ class FakeUpdates(service.AsyncService):
         if builderids is None:
             builderids = []
         # assert types
-        self.testcase.assertIsInstance(scheduler, text_type)
+        self.testcase.assertIsInstance(scheduler, str)
         self.testcase.assertIsInstance(sourcestamps, list)
         for ss in sourcestamps:
             if not isinstance(ss, int) and not isinstance(ss, dict):
-                self.testcase.fail("%s (%s) is not an integer or a dictionary"
-                                   % (ss, type(ss)))
+                self.testcase.fail("{} ({}) is not an integer or a dictionary".format(ss, type(ss)))
             del ss  # since we use locals(), below
-        self.testcase.assertIsInstance(reason, text_type)
+        self.testcase.assertIsInstance(reason, str)
         self.assertProperties(sourced=True, properties=properties)
         self.testcase.assertIsInstance(builderids, list)
         self.testcase.assertIsInstance(external_idstring,
-                                       (type(None), text_type))
+                                       (type(None), str))
 
         self.buildsetsAdded.append(locals())
         self.buildsetsAdded[-1].pop('self')
@@ -176,7 +174,7 @@ class FakeUpdates(service.AsyncService):
         return defer.succeed(None)
 
     @defer.inlineCallbacks
-    def claimBuildRequests(self, brids, claimed_at=None, _reactor=reactor):
+    def claimBuildRequests(self, brids, claimed_at=None):
         validation.verifyType(self.testcase, 'brids', brids,
                               validation.ListValidator(validation.IntValidator()))
         validation.verifyType(self.testcase, 'claimed_at', claimed_at,
@@ -185,7 +183,7 @@ class FakeUpdates(service.AsyncService):
             return True
         try:
             yield self.master.db.buildrequests.claimBuildRequests(
-                brids=brids, claimed_at=claimed_at, _reactor=_reactor)
+                brids=brids, claimed_at=claimed_at)
         except AlreadyClaimedError:
             return False
         self.claimedBuildRequests.update(set(brids))
@@ -199,7 +197,7 @@ class FakeUpdates(service.AsyncService):
         if brids:
             yield self.master.db.buildrequests.unclaimBuildRequests(brids)
 
-    def completeBuildRequests(self, brids, results, complete_at=None, _reactor=reactor):
+    def completeBuildRequests(self, brids, results, complete_at=None):
         validation.verifyType(self.testcase, 'brids', brids,
                               validation.ListValidator(validation.IntValidator()))
         validation.verifyType(self.testcase, 'results', results,
@@ -214,7 +212,7 @@ class FakeUpdates(service.AsyncService):
     def updateBuilderList(self, masterid, builderNames):
         self.testcase.assertEqual(masterid, self.master.masterid)
         for n in builderNames:
-            self.testcase.assertIsInstance(n, text_type)
+            self.testcase.assertIsInstance(n, str)
         self.builderNames = builderNames
         return defer.succeed(None)
 
@@ -304,7 +302,7 @@ class FakeUpdates(service.AsyncService):
         try:
             json.dumps(value)
         except (TypeError, ValueError):
-            self.testcase.fail("Value for %s is not JSON-able" % name)
+            self.testcase.fail("Value for {} is not JSON-able".format(name))
         validation.verifyType(self.testcase, 'source', source,
                               validation.StringValidator())
         return defer.succeed(None)
@@ -433,7 +431,7 @@ class FakeDataConnector(service.AsyncMultiService):
     # relevant updates with fake methods, though.
 
     def __init__(self, master, testcase):
-        service.AsyncMultiService.__init__(self)
+        super().__init__()
         self.setServiceParent(master)
         self.updates = FakeUpdates(testcase)
         self.updates.setServiceParent(self)
