@@ -16,14 +16,13 @@
 import sqlalchemy as sa
 
 from twisted.internet import defer
-from twisted.internet import task
 from twisted.trial import unittest
 
+from buildbot.data import resultspec
 from buildbot.db import builds
 from buildbot.db import changes
 from buildbot.db import sourcestamps
-from buildbot.test.fake import fakedb
-from buildbot.test.fake import fakemaster
+from buildbot.test import fakedb
 from buildbot.test.util import connector_component
 from buildbot.test.util import interfaces
 from buildbot.test.util import validation
@@ -40,7 +39,7 @@ class Tests(interfaces.InterfaceTests):
     change13_rows = [
         fakedb.SourceStamp(id=92, branch="thirteen"),
         fakedb.Change(changeid=13, author="dustin", comments="fix spelling",
-                      branch="master", revision="deadbeef",
+                      branch="master", revision="deadbeef", committer="justin",
                       when_timestamp=266738400, revlink=None, category=None,
                       repository='', codebase='', project='', sourcestampid=92),
 
@@ -54,7 +53,7 @@ class Tests(interfaces.InterfaceTests):
     change14_rows = [
         fakedb.SourceStamp(id=233, branch="fourteen"),
         fakedb.Change(changeid=14, author="warner", comments="fix whitespace",
-                      branch="warnerdb", revision="0e92a098b",
+                      branch="warnerdb", revision="0e92a098b", committer="david",
                       when_timestamp=266738404, revlink='http://warner/0e92a098b',
                       category='devel', repository='git://warner', codebase='mainapp',
                       project='Buildbot', sourcestampid=233),
@@ -66,6 +65,7 @@ class Tests(interfaces.InterfaceTests):
         'changeid': 14,
         'parent_changeids': [],
         'author': 'warner',
+        'committer': 'david',
         'branch': 'warnerdb',
         'category': 'devel',
         'comments': 'fix whitespace',
@@ -84,7 +84,7 @@ class Tests(interfaces.InterfaceTests):
 
     def test_signature_addChange(self):
         @self.assertArgSpecMatches(self.db.changes.addChange)
-        def addChange(self, author=None, files=None, comments=None, is_dir=None,
+        def addChange(self, author=None, committer=None, files=None, comments=None, is_dir=None,
                       revision=None, when_timestamp=None, branch=None, category=None,
                       revlink='', properties=None, repository='', codebase='',
                       project='', uid=None):
@@ -97,10 +97,10 @@ class Tests(interfaces.InterfaceTests):
 
     @defer.inlineCallbacks
     def test_addChange_getChange(self):
-        clock = task.Clock()
-        clock.advance(SOMETIME)
+        self.reactor.advance(SOMETIME)
         changeid = yield self.db.changes.addChange(
             author='dustin',
+            committer='justin',
             files=[],
             comments='fix spelling',
             revision='2d6caa52',
@@ -111,8 +111,7 @@ class Tests(interfaces.InterfaceTests):
             properties={},
             repository='repo://',
             codebase='cb',
-            project='proj',
-            _reactor=clock)
+            project='proj')
         chdict = yield self.db.changes.getChange(changeid)
         validation.verifyDbDict(self, 'chdict', chdict)
         chdict = chdict.copy()
@@ -120,6 +119,7 @@ class Tests(interfaces.InterfaceTests):
         chdict['sourcestampid'] = ss
         self.assertEqual(chdict, {
             'author': 'dustin',
+            'committer': 'justin',
             'branch': 'master',
             'category': None,
             'changeid': changeid,
@@ -154,10 +154,10 @@ class Tests(interfaces.InterfaceTests):
     def test_addChange_withParent(self):
         yield self.insertTestData(self.change14_rows)
 
-        clock = task.Clock()
-        clock.advance(SOMETIME)
+        self.reactor.advance(SOMETIME)
         changeid = yield self.db.changes.addChange(
             author='delanne',
+            committer='melanne',
             files=[],
             comments='child of changeid14',
             revision='50adad56',
@@ -168,8 +168,7 @@ class Tests(interfaces.InterfaceTests):
             properties={},
             repository='git://warner',
             codebase='mainapp',
-            project='Buildbot',
-            _reactor=clock)
+            project='Buildbot')
         chdict = yield self.db.changes.getChange(changeid)
         validation.verifyDbDict(self, 'chdict', chdict)
         chdict = chdict.copy()
@@ -177,6 +176,7 @@ class Tests(interfaces.InterfaceTests):
         chdict['sourcestampid'] = ss
         self.assertEqual(chdict, {
             'author': 'delanne',
+            'committer': 'melanne',
             'branch': 'warnerdb',
             'category': 'devel',
             'changeid': changeid,
@@ -258,14 +258,9 @@ class Tests(interfaces.InterfaceTests):
 
         self.assertEqual(sorted(res), [1, 2])
 
-    def test_signature_getRecentChanges(self):
-        @self.assertArgSpecMatches(self.db.changes.getRecentChanges)
-        def getRecentChanges(self, count):
-            pass
-
     def test_signature_getChanges(self):
         @self.assertArgSpecMatches(self.db.changes.getChanges)
-        def getChanges(self):
+        def getChanges(self, resultSpec=None):
             pass
 
     def insert7Changes(self):
@@ -279,10 +274,11 @@ class Tests(interfaces.InterfaceTests):
         ] + self.change13_rows + self.change14_rows)
 
     @defer.inlineCallbacks
-    def test_getRecentChanges_subset(self):
+    def test_getChanges_subset(self):
         yield self.insert7Changes()
-        changes = yield self.db.changes.getRecentChanges(5)
-
+        rs = resultspec.ResultSpec(order=['-changeid'], limit=5)
+        rs.fieldMapping = {'changeid': 'changes.changeid'}
+        changes = yield self.db.changes.getChanges(resultSpec=rs)
         changeids = [c['changeid'] for c in changes]
         self.assertEqual(changeids, [10, 11, 12, 13, 14])
 
@@ -304,8 +300,9 @@ class Tests(interfaces.InterfaceTests):
         self.assertEqual(n, 100)
 
     @defer.inlineCallbacks
-    def test_getRecentChanges_empty(self):
-        changes = yield self.db.changes.getRecentChanges(5)
+    def test_getChanges_empty(self):
+        rs = resultspec.ResultSpec(order=['-changeid'], limit=5)
+        changes = yield self.db.changes.getChanges(resultSpec=rs)
 
         changeids = [c['changeid'] for c in changes]
         self.assertEqual(changeids, [])
@@ -315,7 +312,7 @@ class Tests(interfaces.InterfaceTests):
         self.assertEqual(changeids, [])
 
     @defer.inlineCallbacks
-    def test_getRecentChanges_missing(self):
+    def test_getChanges_missing(self):
         yield self.insertTestData(self.change13_rows + self.change14_rows)
 
         def check(changes):
@@ -330,7 +327,8 @@ class Tests(interfaces.InterfaceTests):
             self.assertEqual(changes[0]['properties'],
                              {'notest': ('no', 'Change')})
 
-        changes = yield self.db.changes.getRecentChanges(5)
+        rs = resultspec.ResultSpec(order=['-changeid'], limit=5)
+        changes = yield self.db.changes.getChanges(resultSpec=rs)
         check(changes)
 
         changes = yield self.db.changes.getChanges()
@@ -377,10 +375,10 @@ class RealTests(Tests):
 
     @defer.inlineCallbacks
     def test_addChange(self):
-        clock = task.Clock()
-        clock.advance(SOMETIME)
+        self.reactor.advance(SOMETIME)
         changeid = yield self.db.changes.addChange(
             author='dustin',
+            committer='justin',
             files=['master/LICENSING.txt', 'worker/LICENSING.txt'],
             comments='fix spelling',
             revision='2d6caa52',
@@ -391,8 +389,7 @@ class RealTests(Tests):
             properties={'platform': ('linux', 'Change')},
             repository='',
             codebase='cb',
-            project='',
-            _reactor=clock)
+            project='')
         # check all of the columns of the four relevant tables
 
         def thd_change(conn):
@@ -402,6 +399,7 @@ class RealTests(Tests):
             self.assertEqual(len(r), 1)
             self.assertEqual(r[0].changeid, changeid)
             self.assertEqual(r[0].author, 'dustin')
+            self.assertEqual(r[0].committer, 'justin')
             self.assertEqual(r[0].comments, 'fix spelling')
             self.assertEqual(r[0].branch, 'master')
             self.assertEqual(r[0].revision, '2d6caa52')
@@ -460,10 +458,10 @@ class RealTests(Tests):
 
     @defer.inlineCallbacks
     def test_addChange_when_timestamp_None(self):
-        clock = task.Clock()
-        clock.advance(OTHERTIME)
+        self.reactor.advance(OTHERTIME)
         changeid = yield self.db.changes.addChange(
             author='dustin',
+            committer='justin',
             files=[],
             comments='fix spelling',
             revision='2d6caa52',
@@ -474,8 +472,7 @@ class RealTests(Tests):
             properties={},
             repository='',
             codebase='',
-            project='',
-            _reactor=clock)
+            project='')
         # check all of the columns of the four relevant tables
 
         def thd(conn):
@@ -514,6 +511,7 @@ class RealTests(Tests):
         ])
         changeid = yield self.db.changes.addChange(
                           author='dustin',
+                          committer='justin',
                           files=[],
                           comments='fix spelling',
                           revision='2d6caa52',
@@ -611,8 +609,9 @@ class RealTests(Tests):
             for tbl_name in ('scheduler_changes', 'change_files',
                              'change_properties', 'changes'):
                 tbl = self.db.model.metadata.tables[tbl_name]
-                res = conn.execute(sa.select([tbl.c.changeid]))
-                results[tbl_name] = len([row for row in res.fetchall()])
+                res = conn.execute(sa.select([sa.func.count()]).select_from(tbl))
+                results[tbl_name] = res.fetchone()[0]
+                res.close()
             self.assertEqual(results, {
                 'scheduler_changes': 0,
                 'change_files': 0,
@@ -648,7 +647,8 @@ class RealTests(Tests):
 
         codebase_ss = {}  # shared state between addChange and addBuild
 
-        def addChange(codebase, revision, author, comments, branch='master', category='cat', project='proj', repository='repo'):
+        def addChange(codebase, revision, author, committer, comments, branch='master',
+                      category='cat', project='proj', repository='repo'):
             lastID["sourcestampid"] += 1
             lastID["changeid"] += 1
             parent_changeids = codebase_ss.get(codebase, None)
@@ -660,6 +660,7 @@ class RealTests(Tests):
                                              revision=revision),
                           fakedb.Change(changeid=lastID["changeid"],
                                         author=author,
+                                        committer=committer,
                                         comments=comments,
                                         revision=revision,
                                         sourcestampid=lastID["sourcestampid"],
@@ -706,28 +707,28 @@ class RealTests(Tests):
             return buildRows
 
         # Build1 has 1 change per code base
-        rows.extend(addChange('A', 1, 'franck', '1st commit'))
-        rows.extend(addChange('B', 1, 'alice', '2nd commit'))
-        rows.extend(addChange('C', 1, 'bob', '3rd commit'))
+        rows.extend(addChange('A', 1, 'franck', 'franck', '1st commit'))
+        rows.extend(addChange('B', 1, 'alice', 'alice', '2nd commit'))
+        rows.extend(addChange('C', 1, 'bob', 'bob', '3rd commit'))
         rows.extend(addBuild(codebase_ss))
         # Build 2 has only one change for codebase A
-        rows.extend(addChange('A', 2, 'delanne', '4th commit'))
+        rows.extend(addChange('A', 2, 'delanne', 'delanne', '4th commit'))
         rows.extend(addBuild(codebase_ss))
         # Build 3 has only one change for codebase B
-        rows.extend(addChange('B', 2, 'bob', '6th commit'))
+        rows.extend(addChange('B', 2, 'bob', 'bob', '6th commit'))
         rows.extend(addBuild(codebase_ss))
         # Build 4 has no change
         rows.extend(addBuild(codebase_ss))
         # Build 5 has 2 changes for codebase A and 1 change for codebase C
-        rows.extend(addChange('A', 3, 'franck', '7th commit'))
-        rows.extend(addChange('A', 4, 'alice', '8th commit'))
-        rows.extend(addChange('B', 3, 'bob', '9th commit'))
+        rows.extend(addChange('A', 3, 'franck', 'franck', '7th commit'))
+        rows.extend(addChange('A', 4, 'alice', 'alice', '8th commit'))
+        rows.extend(addChange('B', 3, 'bob', 'bob', '9th commit'))
         rows.extend(addBuild(codebase_ss))
         # Build 6 has only one change for codebase C
-        rows.extend(addChange('C', 2, 'bob', '10th commit'))
+        rows.extend(addChange('C', 2, 'bob', 'bob', '10th commit'))
         rows.extend(addBuild(codebase_ss, 2))
         # Build 7 has only one change for codebase C
-        rows.extend(addChange('C', 3, 'bob', '11th commit'))
+        rows.extend(addChange('C', 3, 'bob', 'bob', '11th commit'))
         rows.extend(addBuild(codebase_ss, 2))
         yield self.insertTestData(rows)
 
@@ -746,13 +747,11 @@ class RealTests(Tests):
         yield expect(7, ['11th commit'])
 
 
-class TestFakeDB(unittest.TestCase, Tests):
+class TestFakeDB(unittest.TestCase, connector_component.FakeConnectorComponentMixin, Tests):
 
+    @defer.inlineCallbacks
     def setUp(self):
-        self.master = fakemaster.make_master(wantDb=True, testcase=self)
-        self.db = self.master.db
-        self.db.checkForeignKeys = True
-        self.insertTestData = self.db.insertTestData
+        yield self.setUpConnectorComponent()
 
 
 class TestRealDB(unittest.TestCase,
