@@ -83,7 +83,7 @@ def mTimeVersion(init_file):
     return d.strftime("%Y.%m.%d")
 
 
-def getVersionFromArchiveId(git_archive_id='1579625496  (tag: v2.6.0)'):
+def getVersionFromArchiveId(git_archive_id='1592144576  (tag: v2.8.2, 2.8.x)'):
     """ Extract the tag if a source is from git archive.
 
         When source is exported via `git archive`, the git_archive_id init value is modified
@@ -215,9 +215,8 @@ class BuildJsCommand(distutils.cmd.Command):
             shell = bool(os.name == 'nt')
 
             for command in commands:
-                self.announce(
-                    'Running command: %s' % str(" ".join(command)),
-                    level=distutils.log.INFO)
+                self.announce('Running command: {}'.format(str(" ".join(command))),
+                              level=distutils.log.INFO)
                 subprocess.check_call(command, shell=shell)
 
         self.copy_tree(os.path.join(package, 'static'), os.path.join(
